<?php
class usagereportReport extends reportModel
{
    /**
     * Get usage report data.
     *
     * @param  string $year
     * @param  string $month
     *
     * @access public
     * @return array
     */
    public function getUsageReportData($year = '', $month = '')
    {
        if(empty($year) or empty($month)) return array();

        $report = $this->dao->select('*')->from(TABLE_USAGEREPORT)->where('`year`')->eq($year)->andWhere('`month`')->eq($month)->fetch();
        if(empty($report)) return array();

        $report->activeAccounts       = !empty($report->activeAccounts)       ? json_decode($report->activeAccounts)             : $report->activeAccounts;
        $report->accountDailyActivity = !empty($report->accountDailyActivity) ? json_decode($report->accountDailyActivity)       : $report->accountDailyActivity;
        $report->activeProjects       = !empty($report->activeProjects)       ? json_decode($report->activeProjects)             : $report->activeProjects;
        $report->projectDailyActivity = !empty($report->projectDailyActivity) ? json_decode($report->projectDailyActivity, true) : $report->projectDailyActivity;

        return $report;
    }

    /**
     * Get all usage report.
     *
     * @access public
     * @return array
     */
    public function getUsageReportList()
    {
        return $this->dao->select('`year`, `month`')->from(TABLE_USAGEREPORT)->orderBy('`year` desc, `month` desc')->fetchAll();
    }

    /**
     * Get basic report.
     *
     * @param  string $date
     * @param  object $usageReport
     * @access public
     * @return void
     */
    public function getBasicReport($date, $usageReport)
    {
        list($year, $month, $day) = explode('-', $date);

        /* Get license info. */
        //$ioncubeProperties = array();
        //if(function_exists('ioncube_license_properties'))
        //{
        //    $properties = ioncube_license_properties();
        //    if($properties)
        //    {
        //        foreach($properties as $key => $property) $ioncubeProperties[$key] = $property['value'];
        //    }
        //}
        //if(!empty($ioncubeProperties))
        //{
        //    $licenseType = $ioncubeProperties['expireDate'] == 'All Life' ? 'lifetime' : 'subscribe';
        //    if($ioncubeProperties['expireDate'] != 'All Life')
        //    {
        //        $expireDate = $ioncubeProperties['expireDate'];
        //        $startDate  = $ioncubeProperties['startDate'];
        //        $authTime   = date_diff(date_create($expireDate), date_create($startDate))->days;
        //        if($authTime <= 21) $licenseType = 'try';
        //    }
        //    $usageReport->licenseType      = $licenseType;
        //    $usageReport->licensedAccounts = zget($ioncubeProperties, 'user', 0);
        //}

        /* Get service time. */
        $firstLoginDate = $this->dao->select('date')->from(TABLE_ACTION)->where('objectType')->eq('user')->andWhere('action')->eq('login')->fetch('date');
        $firstLoginDate = empty($firstLoginDate) ? '' : date("Y-m-d", strtotime($firstLoginDate));
        $usageReport->onlineDays = empty($firstLoginDate) ? 0 : date_diff(date_create($date), date_create($firstLoginDate))->days;

        /* Get version. */
        $usageReport->version = $this->config->version;

        /* Get active product count. */
        $productAction = $this->dao->select('id,product')->from(TABLE_ACTION)->where('date')->like("$year-$month-%")->andWhere('product')->notLike("%,0,%")->fetchPairs('id');
        $activeProduct = array();
        foreach($productAction as $products)
        {
            foreach(explode(',', $products) as $productID)
            {
                if(empty($productID)) continue;
                $activeProduct[$productID] = $productID;
            }
        }
        $usageReport->activeProduct = count($activeProduct);

        $projectList = $this->dao->select('*')->from(TABLE_PROJECT)
            ->where('deleted')->eq(0)
            ->andWhere('type')->eq('project')
            ->fetchAll('id');

        /* Get active project count. */
        $activeProject = $this->dao->select('project')->from(TABLE_ACTION)
            ->where('project')->in(array_keys($projectList))
            ->andWhere('date')->like("$year-$month-%")
            ->fetchPairs('project');
        $usageReport->activeProject = empty($projectList) ? 0 : count($activeProject);

        /* Get project model. */
        $projectModel = array();
        foreach($projectList as $id => $project) $projectModel[$project->model] = $project->model;
        $usageReport->projectModel = implode(',', $projectModel);
    }

    /**
     * Get active accounts.
     *
     * @param  string $date
     * @access public
     * @return json
     */
    public function getActiveAccounts($date)
    {
        list($year, $month, $day) = explode('-', $date);
        $active = $this->dao->select('t1.id')->from(TABLE_USER)->alias('t1')
            ->leftJoin(TABLE_ACTION)->alias('t2')->on('t1.id=t2.objectID')
            ->where('t1.deleted')->eq(0)
            ->andWhere('t2.objectType')->eq('user')
            ->andWhere('t2.action')->eq('login')
            ->andWhere('t2.date')->like("$year-$month-%")
            ->beginIF($this->config->vision)->andWhere("CONCAT(',', t1.visions, ',')")->like("%,{$this->config->vision},%")->fi()
            ->groupBy('t2.objectID')
            ->fetchAll('id');
        $active = count($active);

        $total = $this->dao->select('count(1) as count')->from(TABLE_USER)->where('deleted')->eq(0)
            ->beginIF($this->config->vision)->andWhere("CONCAT(',', visions, ',')")->like("%,{$this->config->vision},%")->fi()
            ->fetch('count');

        $ratio = empty($total) ? 0 : sprintf("%.2f", round($active / $total * 100, 2));

        $activeAccounts = new stdClass();
        $activeAccounts->active = $active;
        $activeAccounts->total  = $total;
        $activeAccounts->ratio  = $ratio;

        return json_encode($activeAccounts);
    }

    /**
     * Get daily activity of account.
     *
     * @param  string $date
     * @access public
     * @return json
     */
    public function getAccountDailyActivity($date)
    {
        list($year, $month, $day) = explode('-', $date);

        $actionList = $this->dao->select('id,date,actor')->from(TABLE_ACTION)
            ->where('objectType')->eq('user')
            ->andWhere('action')->eq('login')
            ->andWhere('date')->like("$year-$month-%")
            ->fetchAll('id');

        $loginList = array();
        foreach($actionList as $action)
        {
            $day = substr($action->date, strrpos($action->date, '-') + 1, 2);

            if(!isset($loginList[$day])) $loginList[$day] = array();
            $loginList[$day][$action->actor] = $action->actor;
        }

        $dailyActivity = array();
        for($i = 1; $i <= $day; $i ++) $dailyActivity[$i] = isset($loginList[$i]) ? count($loginList[$i]) : 0;

        return json_encode($dailyActivity);
    }

    /**
     * Get project report.
     *
     * @param  string $date
     * @param  object $usageReport
     * @access public
     * @return void
     */
    public function getProjectReport($date, $usageReport)
    {
        $doingProjects = $this->dao->select('id,name')->from(TABLE_PROJECT)->where('deleted')->eq(0)->andWhere('type')->eq('project')->andWhere('status')->eq('doing')->fetchPairs('id');

        /* Get all doing project team list. */
        $teams = $this->dao->select('t1.root,t1.account')->from(TABLE_TEAM)->alias('t1')
            ->leftJoin(TABLE_USER)->alias('t2')->on('t1.account=t2.account')
            ->where('t1.root')->in(array_keys($doingProjects))
            ->andWhere('t1.type')->eq('project')
            ->andWhere('t2.deleted')->eq(0)
            ->beginIF($this->config->vision)->andWhere("CONCAT(',', t2.visions, ',')")->like("%,{$this->config->vision},%")->fi()
            ->groupBy('t1.root, t1.account')
            ->fetchGroup('root', 'account');
        $accounts = array();
        foreach($teams as $projectID => $team)
        {
            foreach($team as $account => $info) $accounts[$account] = $account;
        }

        /* Get daily account login actions. */
        list($year, $month, $day) = explode('-', $date);
        $actionList = $this->dao->select('id,date,actor')->from(TABLE_ACTION)
            ->where('objectType')->eq('user')
            ->andWhere('actor')->in($accounts)
            ->andWhere('action')->eq('login')
            ->andWhere('date')->like("$year-$month-%")
            ->fetchAll('id');

        $loginList      = array();
        $activeAccounts = array();
        foreach($actionList as $action)
        {
            $day = substr($action->date, strrpos($action->date, '-') + 1, 2);

            if(!isset($loginList[$day])) $loginList[$day] = array();
            $loginList[$day][$action->actor] = $action->actor;
            $activeAccounts[$action->actor]  = $action->actor;
        }

        /* Get active account of project team. */
        $activeProjects = array();
        foreach($teams as $projectID => $team)
        {
            $name   = $doingProjects[$projectID];
            $total  = empty($team) ? 0 : count($team);
            $active = 0;

            foreach($team as $account => $info)
            {
                if(isset($activeAccounts[$account])) $active ++;
            }
            $ratio = empty($total) ? 0 : sprintf("%.2f", round($active / $total * 100, 2));

            if(!isset($activeProjects[$projectID])) $activeProjects[$projectID] = array();
            $activeProjects[$projectID]['name']   = $name;
            $activeProjects[$projectID]['total']  = $total;
            $activeProjects[$projectID]['active'] = $active;
            $activeProjects[$projectID]['ratio']  = $ratio;
        }

        /* Get daily active account of project team. */
        $projectDailyActivity = array();
        foreach($teams as $projectID => $team)
        {
            if(!isset($projectDailyActivity[$projectID])) $projectDailyActivity[$projectID] = array();
            for($i = 1; $i <= $day; $i ++) $projectDailyActivity[$projectID][$i] = isset($loginList[$i]) ? count($loginList[$i]) : 0;
        }

        $usageReport->activeProjects       = json_encode($activeProjects);
        $usageReport->projectDailyActivity = json_encode($projectDailyActivity);
    }
}
